connect scott/tiger
set lines 10000;
set pages 10000;
set long 20000;
set serveroutput on;
SET TRIMSPOOL ON;
SET TAB OFF;
SET FEEDBACK 1;

alter system flush shared_pool;

set echo on;

-- At this point your previous session has crashed due to a problem
-- with our delete statement. We now want to use the SQL Repair Advisor
-- to fit that problem. Before we begin to fix this problem lets ensure
-- that the actual code that prevents this issue in real life is switched off.

pause

alter session set "_fix_control"="5868490:OFF";

-- Before we can run the SQL Repair Advisor we need to create a
-- diagnosis task using dbms_sqldiag. We need to specify the SQL text
-- for the statement that causes the crash and a task name. We also need
-- to specify the problem type. There are three possible problem types

-- PROBLEM_TYPE_COMPILATION_ERROR
-- PROBLEM_TYPE_EXECUTION_ERROR
--  WRONG_RESULTS

-- In this cause since the statement failed during execution we will use
-- PROBLEM_TYPE_EXECUTION_ERROR. When the SQL Repair Advisor completes we will run
-- dbms_sqldiag.report_diagnosis_task to output the results report to the screen.

pause

exec dbms_sqldiag.drop_diagnosis_task('sqldiag_bug_5869490');

variable rep_out clob;

declare
  t_id            varchar2(50);
begin
  t_id := dbms_sqldiag.create_diagnosis_task(
    sql_text => 'delete from t t1 where t1.a = ''a'' and rowid <> (select max(rowid) from t t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d)',
    task_name => 'sqldiag_bug_5869490',
    problem_type =>DBMS_SQLDIAG.PROBLEM_TYPE_COMPILATION_ERROR);

  dbms_sqltune.set_tuning_task_parameter(
    t_id,
    '_SQLDIAG_FINDING_MODE',
    dbms_sqldiag.SQLDIAG_FINDINGS_FILTER_PLANS);

  dbms_output.put_line ('t_id = ' ||  t_id);

  dbms_sqldiag.execute_diagnosis_task (t_id);

  dbms_output.put_line ('executed t_id = ' ||  t_id);

  :rep_out := dbms_sqldiag.report_diagnosis_task (t_id, DBMS_SQLDIAG.TYPE_TEXT);

end;
/

print rep_out

-- The above report tells us that the statement fails during execute
-- and it has found a SQL patch for this statement that will prevent
-- it from crashing in the future. However, unning the repair advisor
-- wont fix the problem automatically. If we run our problematic SQL
-- statement now the session will crash.

pause

delete from t t1 where t1.a = 'a' and rowid <> (select max(rowid) from t t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d);

-- connect scott/tiger

-- We need to accept the SQL Patch that the SQL Repair Advisor recommended before
-- our SQL statement will be able to run without crashing. To accept the  SQL Patch
-- we must execute dbms_sqldiag.accept_sql_patch and pass it task name

pause
execute dbms_sqldiag.accept_sql_patch(task_name => 'sqldiag_bug_5869490', task_owner => 'SCOTT', replace => TRUE);

-- Now that we have successfully accepted the SQL patch we should be able to
-- run our statement without it crashing. Lets first look at the execution
-- plan to see if it has changed.

pause

explain plan for delete from t t1 where t1.a = 'a' and rowid <> (select max(rowid) from t t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d);
-- display the plan
select plan_table_output from table(dbms_xplan.display('plan_table',null,'advanced'));

-- Now lets execute it.

pause

delete from t t1 where t1.a = 'a' and rowid <> (select max(rowid) from t t2 where t1.a= t2.a and t1.b = t2.b and t1.d=t2.d);

-- Finally we need to restore the system back the way we found it.
-- We must drop the SQL Patch we have accepted and turn back on the
-- original code path that prevented this problem in the first place.

--  execute dbms_sqldiag.drop_sql_patch(

alter session set "_fix_control"="5868490:ON";

