#!/bin/bash

export ORACLE_SID=orcl

export ORACLE_HOME=/u01/app/oracle/acfsmounts/acfs_db1

export PATH=/u01/app/oracle/acfsmounts/acfs_db1/bin:/bin:/usr/bin:/usr/local/bin:/usr/X11R6/bin

sqlplus / as sysdba <<FIN!

SET ECHO ON
SET FEEDBACK 1
SET NUMWIDTH 10
SET LINESIZE 8000
SET TRIMSPOOL ON
SET TAB OFF
SET PAGESIZE 100
SET LONG 1000

Rem STS generation for Partition Advisor lab

CONNECT / AS SYSDBA

CREATE OR REPLACE FUNCTION gen_query RETURN VARCHAR2
AUTHID CURRENT_USER IS
result VARCHAR2(400);
rand_query NUMBER;
rand_price NUMBER;
rand_supp NUMBER;
rand_50perc NUMBER;
rand_groupby NUMBER;
rand_climit NUMBER;
BEGIN
 rand_query := DBMS_RANDOM.VALUE();
 rand_price := ROUND(DBMS_RANDOM.VALUE(0,600));
 rand_supp := ROUND(DBMS_RANDOM.VALUE(0,285));
 rand_50perc := DBMS_RANDOM.VALUE();
 rand_groupby := DBMS_RANDOM.VALUE();
 rand_climit := ROUND(DBMS_RANDOM.VALUE(0,15000));
 
 result := 'SELECT';

 IF (rand_query > .75 )
 THEN
 IF (rand_groupby > .5) 
   THEN
     result := result || ' cust_id, COUNT(*)';
   ELSE
     result := result || ' COUNT(*)';
 END IF;
 result := result
        || ' FROM sales s, products p'
        || ' WHERE s.prod_id = p.prod_id'
        || ' AND prod_list_price<'
        ||  rand_price
        || ' AND supplier_id<'
        ||  rand_supp;
 IF (rand_50perc > .5 )
   THEN
     result := result || ' AND   time_id BETWEEN to_date(''01-JAN-1998'')'
                      || ' AND to_date(''31-DEC-1998'')';
 END IF;
 IF (rand_groupby > .5) 
   THEN
     result := result || ' GROUP BY cust_id';
 END IF;

 ELSE -- rand_query
 IF (rand_query > .5)
 THEN
 IF (rand_groupby > .5) 
   THEN
     result := result || ' cust_year_of_birth, COUNT(*)';
   ELSE
     result := result || ' COUNT(*)';
 END IF;
 result := result
        || ' FROM sales s, customers c'
        || ' WHERE s.cust_id = c.cust_id'
        || ' AND cust_credit_limit<'
        ||  rand_climit;
 IF (rand_50perc > .5 )
   THEN
     result := result || ' AND cust_gender = ''M''';
 END IF;
 IF (rand_groupby > .5) 
   THEN
     result := result || ' GROUP BY cust_year_of_birth';
 END IF;
 ELSE -- rand_query
 result := result || ' prod_name';
 result := result
        || ' FROM products p'
        || ' WHERE prod_list_price='
        ||  rand_price
        || ' AND supplier_id='
        ||  rand_supp;

 END IF; -- rand_query
 END IF; -- rand_query
 RETURN result;
END;    
/

show errors

CREATE PUBLIC SYNONYM gen_query FOR gen_query;

GRANT EXECUTE ON gen_query TO sh;

grant dba to sh;

connect sh/sh

set serveroutput on size 32768;
set echo on;
variable norecs number;


Rem Clean up 


declare
    name varchar2(30);
    cursor name_cur1 is
      select task_name from user_advisor_templates
        where task_name like 'SQLACCESS%';
  begin
    ---------------------------------------------------------------------------
    --  Get rid of templates, tasks and workloads.  
    ---------------------------------------------------------------------------

    open name_cur1;
  
    loop
      fetch name_cur1 into name;
      exit when name_cur1%NOTFOUND;
  
      dbms_advisor.update_task_attributes(name,null,null,'FALSE','FALSE');
      dbms_advisor.delete_task(name);
    end loop;
  
    close name_cur1;
  
  end;
/



Rem make a temp table


DROP TABLE temp_table purge;
CREATE TABLE temp_table AS SELECT * FROM SYS.WRI\$_ADV_SQLW_STMTS WHERE NULL IS NOT NULL;



Rem create a large number of pseudo-random (repeatable) queries in the temporary table


alter system flush shared_pool;
execute dbms_sqltune.drop_sqlset('SQLSET_TEST_500');

DECLARE
   sql_stmt   varchar2(1000);
   sqlsetname  VARCHAR2(30);
   sqlsetcur   dbms_sqltune.sqlset_cursor;
   refid    NUMBER;
 k NUMBER := 0;
 num_queries NUMBER := 500;
BEGIN
  DBMS_RANDOM.SEED('Partition Advisor Seed Reset');

  FOR k IN 1..num_queries LOOP
      sql_stmt := gen_query();

      insert into temp_table values(1,1,NULL,0,
      'SH','Access Advisor','Workload',
        0,0,0,0,1,1,2,to_date('02-FEB-2007'),3,0,
        sql_stmt,1);
  END LOOP;

  sqlsetname := 'SQLSET_TEST_500';

  dbms_sqltune.create_sqlset(sqlsetname, 'Generated STS');

   OPEN sqlsetcur FOR
     SELECT 
          SQLSET_ROW(null,null, sql_text, null, null, username, module,
                     action, elapsed_time, cpu_time, buffer_gets, disk_reads, 
                     0,rows_processed, 0, executions, 0, optimizer_cost, null, 
                     priority, command_type, 
                     to_char(last_execution_date,'yyyy-mm-dd/hh24:mi:ss'),
                     0,0,NULL,0,NULL,NULL
                     )
    FROM temp_table; 

   dbms_sqltune.load_sqlset(sqlsetname, sqlsetcur);
END;
/


SELECT COUNT(*) FROM TABLE(DBMS_SQLTUNE.SELECT_SQLSET('SQLSET_TEST_500'));




Rem Cleanup anything left behind


execute dbms_advisor.delete_task('%');
execute dbms_advisor.delete_sqlwkld('%');

variable     l_task_id number;
variable     l_name varchar2(30);


Rem make a new template to have a minimum of 100 rows only for partitioning

begin
:l_name := 'SQLACCESS_PARTITION_SMALL';
end;
/

begin
 dbms_advisor.update_task_attributes(:l_name,null,null,'FALSE');
 dbms_advisor.delete_task(:l_name);
end;
/

begin
 dbms_advisor.create_task(dbms_advisor.sqlaccess_advisor,:l_task_id,:l_name,'For demo purposes only');
 dbms_advisor.set_task_parameter(:l_name,'_PA_MIN_TABLE_SIZE_PART',100);
 dbms_advisor.set_task_parameter(:l_name,'EXECUTION_TYPE','FULL');
 dbms_advisor.set_task_parameter(:l_name,'ANALYSIS_SCOPE','ALL');
 dbms_advisor.set_task_parameter(:l_name,'MODE','COMPREHENSIVE');

 dbms_advisor.update_task_attributes(:l_name,null,null,'TRUE','TRUE');
end;
/


conn / as sysdba

DROP PUBLIC SYNONYM gen_query;
DROP FUNCTION gen_query;

REVOKE DBA from SH;

GRANT ADVISOR to SH;
GRANT OEM_ADVISOR to SH;
GRANT ADMINISTER SQL TUNING SET to SH;

exit;

FIN!

